@@ -2,7 +2,7 @@
  * =========================================================================
  * This file is part of the GrInvIn project - http://www.grinvin.org
  * 
- * Copyright (C) 2005-2007 Universiteit Gent
+ * Copyright (C) 2005-2008 Universiteit Gent
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,10 +29,11 @@ package org.grinvin.invariants.computers.standard;
 
 import java.util.ArrayList;
 import java.util.Arrays;
-import org.grinvin.Edge;
-import org.grinvin.GraphBundleView;
-import org.grinvin.invariants.IntegerValue;
+
+import org.grinvin.graphs.Edge;
+import org.grinvin.graphs.GraphBundleView;
 import org.grinvin.invariants.computers.AbstractInvariantComputer;
+import org.grinvin.invariants.values.IntegerValue;
 
 /**
  * computes the domination number of the graph.
