@@ -2,7 +2,7 @@
  * =========================================================================
  * This file is part of the GrInvIn project - http://www.grinvin.org
  * 
- * Copyright (C) 2005-2007 Universiteit Gent
+ * Copyright (C) 2005-2008 Universiteit Gent
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,14 +27,13 @@
 
 package org.grinvin.invariants.computers.standard.matching;
 
-import java.util.HashMap;
-import java.util.Iterator;
-import java.util.LinkedList;
 import java.util.List;
+import java.util.ArrayList;
+import java.util.LinkedList;
+import java.util.Iterator;
 import java.util.ListIterator;
 import java.util.Set;
 import java.util.HashSet;
-import java.util.Map;
 
 /**
  * This class computes the matching number(maximum size of matching) of a
@@ -44,13 +43,13 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
     
     // PRIVATE VARIABLES
     
-    /** HashMap to represent the graph. */
-    private Map<Integer,PseudoVertex> graph;
+    /** ArrayList to represent the graph. */
+    private ArrayList<PseudoVertex> graph;
     /** Set of vertices reached along edges in maxMatching. */
-    private Set<PseudoVertex> s;
+    private HashSet<PseudoVertex> s;
     /** Set of vertices reached along edges not in maxMatching. */
-    private Set<PseudoVertex> t;
-    /** List with unexplored vertices in S. */
+    private HashSet<PseudoVertex> t;
+    /** Queue with unexplored vertices in S. */
     private LinkedList<PseudoVertex> unexplored;
     
     /**
@@ -70,17 +69,20 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
      */
     public int compute(int[][] adjlist) {
         super.adjlist = adjlist;
-        initializeMatching();
+        initializeMatching(DEGREE_GREEDY_HEUR);
+        int unmatched = adjlist.length-(2*matchingSize);
+        if(unmatched < 2) {
+            return matchingSize;
+        }
         // Build (original) graph
         initializeGraph();
         // Initialize some aiding variables
         s = new HashSet<PseudoVertex>();
         t = new HashSet<PseudoVertex>();
-        unexplored = new LinkedList<PseudoVertex>(); // ToDo: linkedlist?!!!
-        int maxMatchingSize = adjlist.length/2;
-        for(int u = 0; u < adjlist.length && matchingSize < maxMatchingSize; u++) {
+        unexplored = new LinkedList<PseudoVertex>(); // Queue
+        for(int u = 0; u < adjlist.length && unmatched >= 2; u++) {
             if(matching[u] == UNSATURATED) { // is u unsaturated?
-                PseudoVertex uVertex = graph.get(new Integer(u));
+                PseudoVertex uVertex = graph.get(u);
                 uVertex.setParent(null); // Root of alternating tree!
                 s.clear();
                 s.add(uVertex);
@@ -90,7 +92,9 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
                 PseudoVertex lastNode = findAugmentingPath();
                 if(lastNode != null) {
                     updateMaxMatching(lastNode);
+                    unmatched = unmatched - 2;
                 }
+                // else...clean-up heuristic: not used!
                 cleanUpBlossoms();
             }
         }
@@ -98,16 +102,15 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
     }
     
     /**
-     * Initializes the graph. A graph is represented as a HashMap.
-     * The key is the id-number(the index in the adjacencylist) and
-     * the value is the corresponding PseudoVertex-object.
+     * Initializes the graph. A graph is represented as a ArrayList
+     * of PseudoVertex-objects.
      */
     private void initializeGraph() {
-        graph = new HashMap<Integer,PseudoVertex>();
+        graph = new ArrayList<PseudoVertex>();
         for(int i = 0; i < adjlist.length; i++)
-            graph.put(new Integer(i), new PseudoVertex(i));
+            graph.add(new PseudoVertex(i));
         for(int i = 0; i < adjlist.length; i++) {
-            PseudoVertex v = graph.get(new Integer(i));
+            PseudoVertex v = graph.get(i);
             if(matching[i] != UNSATURATED)
                 v.setSaturated(true);
             for(int j = 0; j < adjlist[i].length; j++)
@@ -122,15 +125,14 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
      */
     private PseudoVertex findAugmentingPath() {
         int highestIndex = adjlist.length; // For 'new' vertices: blossoms
-        PseudoVertex selected = unexplored.peek();
+        PseudoVertex root = unexplored.peek();
         while(!unexplored.isEmpty()) {
             PseudoVertex v = unexplored.poll();
             Iterator<PseudoVertex> it = v.getNeighbours().iterator();
-            boolean blossomFound = false;
-            while(it.hasNext() && !blossomFound) { // For all neighbours of v
+            while(it.hasNext()) { // For all neighbours of v (via unsaturated edge)
                 PseudoVertex neighbour = it.next();
-                if(unsaturatedEdge(v, neighbour) && !t.contains(neighbour)
-                && !blossomFound && neighbour != selected) { // Via unsaturated edge!
+                if(neighbour != root && unsaturatedEdge(v, neighbour) 
+                && !t.contains(neighbour)) {
                     if(neighbour.isSaturated()) {
                         if(!s.contains(neighbour)) {
                             neighbour.setParent(v);
@@ -140,8 +142,8 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
                             s.add(w);
                             unexplored.add(w);
                         } else {
-                            blossomFound = true;
                             contractBlossom(v, neighbour, highestIndex++);
+                            break;
                         }
                     }
                     else { // found augmenting path!
@@ -155,10 +157,10 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
     }
     
     /**
-     * This method searches for the incident vertex of the specified
-     * saturated vertex in the current matching.
-     * @param vertex A saturated vertex.
-     * @return The incident vertex in the matching.
+     * This method searches for the other saturated vertex of 
+     * a saturated edge by a given saturated vertex.
+     * @param vertex The first saturated vertex.
+     * @return The other saturated vertex.
      */
     private PseudoVertex searchIncidentVertex(PseudoVertex vertex) {
         if(vertex.isBlossom())
@@ -200,7 +202,6 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
      */
     private void contractBlossom(PseudoVertex first, PseudoVertex scd, 
                                  int highestIndex) {
-        Integer hIndex = new Integer(highestIndex);
         Set<PseudoVertex> set = new HashSet<PseudoVertex>();
         LinkedList<PseudoVertex> bList = new LinkedList<PseudoVertex>();
         // Explore both paths to the vertex and discover the correct base
@@ -218,7 +219,7 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
         b.setSaturated(true);
         s.add(b);
         unexplored.addFirst(b);
-        graph.put(hIndex, b);
+        graph.add(b);
     }
     
     /**
@@ -300,7 +301,6 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
      */
     private PseudoVertex expandBlossom(PseudoVertex blossom, PseudoVertex child) {
         LinkedList<PseudoVertex> bList = blossom.expandBlossom();
-        graph.remove(blossom.getId());
         // Search for the real parent of the child in the blossom
         int parentIndex = findRealParent(blossom, child);
         PseudoVertex newParent = bList.get(parentIndex);
@@ -326,7 +326,6 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
         else {
             updatePath(bList, blossom, startIndex, stopIndex, correctPathDirection);
         }
-        expandRemainingBlossoms(bList);
         return newParent;
     }
     
@@ -405,9 +404,10 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
         boolean parentFound = false;
         boolean unsaturatedEdge = unsaturatedEdge(blossom, child);
         int parentIndex = 0; // Index of the new parent of child in the blossomlist
+        HashSet<PseudoVertex> neighbours = child.getNeighbours();
         while(lit.hasNext() && !parentFound) {
             PseudoVertex vertex = lit.next();
-            if(vertex.getNeighbours().contains(child) &&
+            if(neighbours.contains(vertex) &&
                (unsaturatedEdge(vertex, child)==unsaturatedEdge)) {
                 parentFound = true;
                 parentIndex = lit.previousIndex();
@@ -438,31 +438,46 @@ public class NonBipartiteMatchingNumber extends GeneralMatchingNumber {
     }
     
     /**
-     * This recursive method expands remaining blossoms in the blossomlist of 
-     * a blossom after updating the path with the method updatePath().
-     * @param bList The blossomList with the remaining vertices from the list.
+     * Removes all the blossoms from the graph.
      */
-    private void expandRemainingBlossoms(LinkedList<PseudoVertex> bList) {
-        for(PseudoVertex vertex : bList) {
-            if(vertex.isBlossom()) {
-                graph.remove(vertex.getId());
-                expandRemainingBlossoms(vertex.expandBlossom());
-            }
+    private void cleanUpBlossoms() {
+        int index = graph.size()-1;
+        while(index >= adjlist.length) {
+            PseudoVertex blossom = graph.get(index);
+            blossom.expandBlossom();
+            graph.remove(index);
+            index--;
         }
     }
     
     /**
-     * Clean up all the remaining blossoms!
+     * This method checks all neighbours of the specified vertex
+     * and looks for unsaturated neighbours.
+     * @param v The vertex to examine.
+     * @param u The root of the tree.
+     * @return True if the given vertex has an unsaturated neighbour
      */
-    private void cleanUpBlossoms() {
-        int index = adjlist.length;
-        while(graph.size() > adjlist.length) {
-            PseudoVertex blossom = graph.get(index);
-            if(blossom != null) {
-                expandRemainingBlossoms(blossom.expandBlossom());
-                graph.remove(index);
+    private boolean lookAheadHeuristic(PseudoVertex v, PseudoVertex root) {
+        for(PseudoVertex n : v.neighbours) {
+            if(!n.isSaturated() && n!=root && !t.contains(n)) {
+                return true;
+            }
+        }
+        return false;
+    }
+    
+    /**
+     * This method deletes all vertices visited during a search for an
+     * augmenting path if the search failed.
+     */
+    private void cleanUpHeuristic() {
+        ListIterator<PseudoVertex> lit = graph.listIterator();
+        while(lit.hasNext() && lit.nextIndex() != adjlist.length) {
+            PseudoVertex v = lit.next();
+            if(s.contains(v) || t.contains(v)) {
+                v.removeFromGraph();
             }
-            index++;
         }
     }
+
 }
